@@ -1,28 +1,39 @@
 import asyncio
-from typing import Any, AsyncIterator, Dict, Optional, Tuple
+from typing import Any, AsyncIterator, Dict, Mapping, Optional, Tuple, Type, Union
+
+from redis.asyncio import BlockingConnectionPool
+
+from cashews.backends.interface import Backend
 
-from ..interface import Backend
 from .client import Redis, SafeRedis
 
-try:
-    from redis.asyncio import BlockingConnectionPool
-except ImportError:
-    from aioredis import BlockingConnectionPool
-
-
 _UNLOCK = """
-if redis.call("get",KEYS[1]) == ARGV[1] then
-    return redis.call("del",KEYS[1])
+if redis.call("GET", KEYS[1]) == ARGV[1] then
+    return redis.call("DEL", KEYS[1])
 else
     return 0
 end
 """
+_INCR_SLICE = """
+redis.call("ZREMRANGEBYSCORE", KEYS[1], 0, "(" .. ARGV[1])
+local current_count = redis.call("ZCOUNT", KEYS[1], ARGV[1], ARGV[2])
+if current_count < tonumber(ARGV[3]) then
+    current_count = current_count + 1
+    redis.call("ZADD", KEYS[1], ARGV[2], ARGV[2])
+    if tonumber(ARGV[4]) > 0 then
+        redis.call("PEXPIRE", KEYS[1], ARGV[4])
+    end
+end
+return current_count
+"""
+_empty = object()
 # pylint: disable=arguments-differ
 # pylint: disable=abstract-method
 
 
 class _Redis(Backend):
-    name = "redis"
+    _client: Union[Redis, SafeRedis]
+    _client_class: Union[Type[Redis], Type[SafeRedis]]
 
     def __init__(self, address, safe: bool = True, **kwargs: Any) -> None:
         kwargs.pop("local_cache", None)
@@ -38,9 +49,11 @@ class _Redis(Backend):
         self._pool_class = kwargs.pop("connection_pool_class", BlockingConnectionPool)
         if self._pool_class == BlockingConnectionPool:
             kwargs["timeout"] = kwargs.pop("wait_for_connection_timeout", 0.1)
-        self._client = None
-        self._sha = {}
-        self._safe = safe
+        self._sha: Dict[str, Any] = {}
+        if not safe:
+            self._client_class = Redis
+        else:
+            self._client_class = SafeRedis
         self._kwargs = kwargs
         self._address = address
         self.__is_init = False
@@ -51,11 +64,7 @@ class _Redis(Backend):
         return self.__is_init
 
     async def init(self):
-        if not self._safe:
-            client_class = Redis
-        else:
-            client_class = SafeRedis
-        self._client = client_class(connection_pool=self._pool_class.from_url(self._address, **self._kwargs))
+        self._client = self._client_class(connection_pool=self._pool_class.from_url(self._address, **self._kwargs))
         await self._client.initialize()
         self.__is_init = True
 
@@ -74,7 +83,7 @@ class _Redis(Backend):
             expire = None
         return bool(await self._client.set(key, value, ex=expire, px=pexpire, nx=nx, xx=xx))
 
-    async def set_many(self, pairs: Dict[str, Any], expire: Optional[float] = None):
+    async def set_many(self, pairs: Mapping[str, Any], expire: Optional[float] = None):
         await self._client.mset(pairs)
         if expire is not None:
             async with self._client.pipeline(transaction=True) as pipe:
@@ -85,8 +94,8 @@ class _Redis(Backend):
     async def get_expire(self, key: str) -> int:
         return await self._client.ttl(key)
 
-    async def expire(self, key: str, timeout: Optional[float]) -> int:
-        return await self._client.expire(key, int(timeout))
+    async def expire(self, key: str, timeout: float):
+        return await self._client.pexpire(key, int(timeout * 1000))
 
     async def set_lock(self, key: str, value, expire: float) -> bool:
         pexpire = None
@@ -117,45 +126,51 @@ class _Redis(Backend):
         return await self._client.evalsha(self._sha["UNLOCK"], 1, key, value)
 
     async def delete(self, key: str) -> bool:
-        return await self._client.unlink(key)
+        return bool(await self._client.unlink(key))
 
     async def exists(self, key: str) -> bool:
         return bool(await self._client.exists(key))
 
-    async def scan(self, pattern: str, batch_size: int = 100) -> AsyncIterator[str]:
-        cursor = b"0"
-        while cursor:
+    async def scan(self, pattern: str, batch_size: int = 100) -> AsyncIterator[str]:  # type: ignore
+        cursor = 0
+        while True:
             cursor, keys = await self._client.scan(cursor, match=pattern, count=batch_size)
             for key in keys:
                 yield key.decode()
+            if not cursor:
+                return
 
-    async def keys_match(self, pattern: str):
-        async for key in self.scan(pattern):
-            yield key.encode()
+    async def delete_many(self, *keys: str):
+        await self._client.delete(*keys)
 
     async def delete_match(self, pattern: str):
         if "*" not in pattern:
-            return await self._client.unlink(pattern)
-        cursor = b"0"
-        while cursor:
+            await self._client.unlink(pattern)
+            return
+        cursor = 0
+        while True:
             cursor, keys = await self._client.scan(cursor, match=pattern, count=100)
             if not keys:
+                if not cursor:
+                    return
                 continue
             await self._client.unlink(keys[0], *keys[1:])
-        return True
 
-    async def get_match(
-        self, pattern: str, batch_size: int = 100, default: Optional[Any] = None
-    ) -> AsyncIterator[Tuple[str, bytes]]:
-        cursor = b"0"
-        while cursor:
+    async def get_match(self, pattern: str, batch_size: int = 100) -> AsyncIterator[Tuple[str, Any]]:  # type: ignore
+        cursor = 0
+        while True:
             cursor, keys = await self._client.scan(cursor, match=pattern, count=batch_size)
             if not keys:
+                if not cursor:
+                    return
                 continue
             keys = [key.decode() for key in keys]
-            values = await self.get_many(*keys, default=default)
+            values = await self.get_many(*keys, default=_empty)
             for key, value in zip(keys, values):
-                yield key, value
+                if value is not _empty:  # key can be deleted after scan
+                    yield key, value
+            if not cursor:
+                return
 
     async def get_size(self, key: str) -> int:
         size = await self._client.memory_usage(key) or 0
@@ -164,7 +179,7 @@ class _Redis(Backend):
     async def get(self, key: str, default=None) -> Any:
         return await self._client.get(key)
 
-    async def get_many(self, *keys: str, default: Optional[Any] = None) -> Tuple[Optional[Any]]:
+    async def get_many(self, *keys: str, default: Optional[Any] = None) -> Tuple[Optional[Any], ...]:
         if not keys:
             return tuple()
         values = await self._client.mget(keys[0], *keys[1:])
@@ -184,7 +199,7 @@ class _Redis(Backend):
             bitops.get(fmt=f"u{size}", offset=f"#{index}")
         return tuple(await bitops.execute() or [])
 
-    async def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1):
+    async def incr_bits(self, key: str, *indexes: int, size: int = 1, by: int = 1) -> Tuple[int, ...]:
         bitops = self._client.bitfield(key)
         for index in indexes:
             bitops.incrby(fmt=f"u{size}", offset=f"#{index}", increment=by, overflow="SAT")
@@ -202,6 +217,13 @@ class _Redis(Backend):
     async def get_raw(self, key: str) -> Any:
         return await self._client.get(key)
 
-    def close(self):
-        self._client = None
+    async def slice_incr(self, key: str, start: int, end: int, maxvalue: int, expire: Optional[float] = None) -> int:
+        expire = expire or 0
+        expire = int(expire * 1000)
+        if "INCR_SLICE" not in self._sha:
+            self._sha["INCR_SLICE"] = await self._client.script_load(_INCR_SLICE.replace("\n", " "))
+        return await self._client.evalsha(self._sha["INCR_SLICE"], 1, key, start, end, maxvalue, expire)
+
+    async def close(self):
+        await self._client.close()
         self.__is_init = False
