@@ -1,4 +1,5 @@
-from .._cache_condition import NOT_NONE
+from cashews.cache_condition import NOT_NONE  # noqa
+
 from .bloom import bloom, dual_bloom  # noqa
 from .cache.defaults import CacheDetect, context_cache_detect  # noqa
 from .cache.early import early  # noqa
@@ -6,6 +7,7 @@ from .cache.fail import failover, fast_condition  # noqa
 from .cache.hit import hit  # noqa
 from .cache.simple import cache  # noqa
 from .cache.soft import soft  # noqa
-from .circuit_breaker import CircuitBreakerOpen, circuit_breaker  # noqa
+from .circuit_breaker import circuit_breaker  # noqa
 from .locked import locked  # noqa
-from .rate import RateLimitException, rate_limit  # noqa
+from .rate import rate_limit  # noqa
+from .rate_slide import slice_rate_limit  # noqa
