@@ -1,11 +1,13 @@
 from functools import wraps
 from typing import Optional, Tuple, Type, Union
 
-from ..._typing import CallableCacheCondition
-from ...backends.interface import Backend
-from ...formatter import register_template
-from ...key import get_cache_key, get_cache_key_template
-from .defaults import CacheDetect, _empty, context_cache_detect
+from cashews._typing import TTL, AsyncCallable_T, CallableCacheCondition, Decorator
+from cashews.backends.interface import _BackendInterface
+from cashews.formatter import register_template
+from cashews.key import get_cache_key, get_cache_key_template
+from cashews.ttl import ttl_to_seconds
+
+from .defaults import _empty, context_cache_detect
 
 __all__ = ("failover",)
 
@@ -22,13 +24,13 @@ def fast_condition(getter, setter=None):
 
 
 def failover(
-    backend: Backend,
-    ttl: int,
+    backend: _BackendInterface,
+    ttl: TTL,
     exceptions: Union[Type[Exception], Tuple[Type[Exception]]] = Exception,
     key: Optional[str] = None,
     condition: CallableCacheCondition = lambda *args, **kwargs: True,
     prefix: str = "fail",
-):
+) -> Decorator:
     """
     Return cache result (at list 1 call of function call should be succeed) if call raised one of given exception,
     :param backend: cache backend
@@ -39,21 +41,24 @@ def failover(
     :param prefix: custom prefix for key, default "fail"
     """
 
-    def _decor(func):
+    ttl = ttl_to_seconds(ttl)
+
+    def _decor(func: AsyncCallable_T) -> AsyncCallable_T:
         _key_template = get_cache_key_template(func, key=key, prefix=prefix)
         register_template(func, _key_template)
 
         @wraps(func)
-        async def _wrap(*args, _from_cache: CacheDetect = context_cache_detect, **kwargs):
+        async def _wrap(*args, **kwargs):
+            _ttl = ttl_to_seconds(ttl, *args, **kwargs, with_callable=True)
             _cache_key = get_cache_key(func, _key_template, args, kwargs)
             try:
                 result = await func(*args, **kwargs)
             except exceptions as exc:
                 cached = await backend.get(_cache_key, default=_empty)
                 if cached is not _empty:
-                    _from_cache._set(
+                    context_cache_detect._set(
                         _cache_key,
-                        ttl=ttl,
+                        ttl=_ttl,
                         exc=exc,
                         name="failover",
                         template=_key_template,
@@ -62,7 +67,7 @@ def failover(
                 raise exc
             else:
                 if condition(result, args, kwargs, key=_cache_key):
-                    await backend.set(_cache_key, result, expire=ttl)
+                    await backend.set(_cache_key, result, expire=_ttl)
                 return result
 
         return _wrap
