@@ -1,64 +1,29 @@
 import inspect
-from datetime import timedelta
 from functools import lru_cache
-from typing import Any, Callable, Container, Dict, Optional, Tuple, Union
+from typing import Any, Callable, Container, Dict, Optional, Tuple
 
-from ._typing import TTL
+from .exceptions import WrongKeyError
 from .formatter import _ReplaceFormatter, default_formatter, template_to_pattern
 
 _KWARGS = "__kwargs__"
 _ARGS = "__args__"
 _ARGS_KWARGS = (_ARGS, _KWARGS)
-
-
-class WrongKeyException(ValueError):
-    """Raised If key template have wrong parameter"""
-
-
-def ttl_to_seconds(ttl: Union[float, None, TTL]) -> Union[int, None, float]:
-    timeout = ttl() if callable(ttl) else ttl
-    if isinstance(timeout, timedelta):
-        return timeout.total_seconds()
-    if isinstance(timeout, str):
-        return _ttl_from_str(timeout)
-    return timeout
-
-
-_STR_TO_DELTA = {
-    "h": timedelta(hours=1),
-    "m": timedelta(minutes=1),
-    "s": timedelta(seconds=1),
-    "d": timedelta(days=1),
-}
-
-
-def _ttl_from_str(ttl: str) -> int:
-    result = 0
-    mul = ""
-    for char in ttl.strip().lower():
-        if char.isdigit():
-            mul += char
-        elif char in _STR_TO_DELTA:
-            result += int(mul) * _STR_TO_DELTA[char].total_seconds()
-            mul = ""
-        else:
-            raise ValueError(f"ttl '{ttl}' has wrong string representation")
-    if mul != "" and not result:
-        return int(mul)
-    return result
+Args = Tuple[Any, ...]
+Kwargs = Dict[str, Any]
 
 
 def get_cache_key(
     func: Callable,
     template: Optional[str] = None,
-    args: Tuple[Any] = (),
-    kwargs: Optional[Dict] = None,
+    args: Args = (),
+    kwargs: Optional[Kwargs] = None,
 ) -> str:
     """
     Get cache key name for function (:param func) called with args and kwargs
     if func_args is passed key build with parameters are included in func_args dict or tuple otherwise use all of them
     Used function module and name as prefix if key parameter not passed
     :param func: Target function
+    :param template: precompile template
     :param args: call positional arguments
     :param kwargs: call keyword arguments
     :return: cache key for call
@@ -69,7 +34,7 @@ def get_cache_key(
     return template_to_pattern(_key_template, _formatter=default_formatter, **key_values)
 
 
-def get_func_params(func):
+def get_func_params(func: Callable):
     signature = _get_func_signature(func)
     for param_name, param in signature.parameters.items():
         if param.kind == inspect.Parameter.VAR_KEYWORD:
@@ -80,6 +45,7 @@ def get_func_params(func):
             yield param_name
 
 
+@lru_cache(maxsize=10000)
 def get_cache_key_template(
     func: Callable, key: Optional[str] = None, prefix: str = "", exclude_parameters: Container = ()
 ) -> str:
@@ -88,6 +54,7 @@ def get_cache_key_template(
     Used function module and name as prefix if key parameter not passed
     :param func: Target function
     :param key: template for key, may contain alias to args or kwargs passed to a call
+    :param prefix: a prefix
     :param exclude_parameters: array of `args` and `kwargs` names to exclude from a key
         template (if key parameter not passed)
     :return: cache key template
@@ -111,7 +78,7 @@ def generate_key_template(func: Callable, exclude_parameters: Container = ()):
         from a key template (if key parameter not passed)
     :return: cache key template
     """
-    func_params = list(get_func_params(func))
+    func_params = tuple(get_func_params(func))
     key_template = f"{func.__module__}:{func.__name__}"
     if func_params and func_params[0] == "self":
         key_template = f"{func.__module__}:{func.__qualname__}"
@@ -144,10 +111,10 @@ def _check_key_params(key, func_params):
     check = _ReplaceFormatter(default=_default)
     check.format(key, **func_params)
     if errors:
-        raise WrongKeyException(f"Wrong parameter placeholder '{errors}' in the key ")
+        raise WrongKeyError(f"Wrong parameter placeholder '{errors}' in the key ")
 
 
-def get_call_values(func: Callable, args, kwargs) -> Dict:
+def get_call_values(func: Callable, args: Args, kwargs: Kwargs) -> Dict:
     """
     Return dict with arguments and their values for function call with given positional and keywords arguments
     :param func: Target function
@@ -161,12 +128,20 @@ def get_call_values(func: Callable, args, kwargs) -> Dict:
     return key_values
 
 
-@lru_cache(maxsize=100)
-def _get_func_signature(func):
+@lru_cache(maxsize=1000)
+def _get_func_signature(func: Callable):
     return inspect.signature(func)
 
 
-def _get_call_values(func, args, kwargs):
+def _get_call_values(func: Callable, args: Args, kwargs: Kwargs):
+    if len(args) == 0:
+        _kwargs = {**kwargs}
+        for name, parameter in _get_func_signature(func).parameters.items():
+            if parameter.kind != inspect.Parameter.VAR_KEYWORD:
+                if name in _kwargs:
+                    del _kwargs[name]
+        return {**kwargs, _KWARGS: _kwargs}
+
     signature = _get_func_signature(func).bind(*args, **kwargs)
     signature.apply_defaults()
     result = {}
@@ -186,7 +161,7 @@ def noself(decor_func):
     def _decor(*args, **kwargs):
         def outer(method):
             if "key" not in kwargs:
-                kwargs["key"] = get_cache_key_template(method, exclude_parameters={"self"})
+                kwargs["key"] = get_cache_key_template(method, exclude_parameters=("self",))
             return decor_func(*args, **kwargs)(method)
 
         return outer
