@@ -3,4 +3,4 @@ try:
 except ImportError:
     from ._bitarray import Bitarray
 from .object_size import get_obj_size
-from .split_hash import get_hashes
+from .split_hash import get_indexes
