@@ -4,7 +4,6 @@ import pytest
 import pytest_asyncio
 
 from cashews.backends.memory import Memory
-from cashews.decorators.bloom import _counting_bloom as counting_bloom
 from cashews.decorators.bloom import bloom, dual_bloom
 
 pytestmark = pytest.mark.asyncio
@@ -99,30 +98,3 @@ async def test_bloom_dual(backend):
     call.reset_mock()
     assert await func(70) is True
     call.assert_called_with(70)
-
-
-async def test_bloom_counting(backend):
-    n = 100
-    call = Mock()
-
-    @counting_bloom(backend=backend, name="name:{k}", false_positives=0.1, capacity=n)
-    async def func(k):
-        call(k)
-        return k > (n / 2)
-
-    for i in range(n):
-        await func.set(i)
-
-    call.reset_mock()
-    for i in range(n):
-        assert await func(i) is (i > (n / 2))
-
-    await func.delete(10)
-    await func.delete(70)
-
-    assert await func(10) is False
-    call.assert_called_with(10)
-
-    call.reset_mock()
-    assert await func(70) is True
-    call.assert_called_with(70)
