@@ -2,7 +2,7 @@ from unittest.mock import Mock
 
 import pytest
 
-from cashews.backends.interface import CacheBackendInteractionException
+from cashews import CacheBackendInteractionError
 from cashews.wrapper import Cache
 
 pytestmark = [pytest.mark.asyncio, pytest.mark.redis]
@@ -36,35 +36,34 @@ async def test_safe_redis(redis_backend):
 
     assert await redis.get_expire("test") == 0
     assert await redis.incr("test") is None
+    assert await redis.slice_incr("test", 1, 2, 10) is None
     assert await redis.get_size("test") == 0
     async for _ in redis.scan("*"):
         assert False
 
     async for _ in redis.get_match("*"):
         assert False
-    async for _ in redis.keys_match("*"):
-        assert False
 
     assert await redis.delete("test") == 0
     await redis.delete_match("*")
 
-    with pytest.raises(CacheBackendInteractionException):
+    with pytest.raises(CacheBackendInteractionError):
         await redis.ping()
 
 
 async def test_unsafe_redis_down(redis_backend):
     redis = redis_backend(safe=False, address="redis://localhost:9223", hash_key=None)
     await redis.init()
-    with pytest.raises(CacheBackendInteractionException):
+    with pytest.raises(CacheBackendInteractionError):
         await redis.ping()
-    with pytest.raises(CacheBackendInteractionException):
+    with pytest.raises(CacheBackendInteractionError):
         await redis.set("key", "value")
 
 
 async def test_cache_decorators_on_redis_down(redis_backend):
     mock = Mock(return_value="val")
     cache = Cache()
-    cache._add_backend(redis_backend, safe=True, address="redis://localhost:9223", hash_key=None)
+    cache._add_backend(redis_backend(safe=True, address="redis://localhost:9223", hash_key=None))
 
     @cache(ttl=1)
     @cache.failover(1)
